#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include "DCTag.h"

static void print_tag_value ( char const * tag_name, DC_TAG * tag_value )
{
	time_t tt;
	struct tm * gmt;

	tt = tag_value->time;
	gmt = gmtime( &tt );

	printf(
		"%s, value = %lf, quality = %04xH, timestamp_UTC = %04d-%02d-%02d %02d:%02d:%02d (%llu.%06u)\n",
		tag_name,
		tag_value->value,
		(uint16_t) tag_value->quality,
		1900 + gmt->tm_year,
		gmt->tm_mon + 1,
		gmt->tm_mday,
		gmt->tm_hour,
		gmt->tm_min,
		gmt->tm_sec,
		tag_value->time,
		tag_value->usec );
}

static int read_tag (
	char const * tag_name,
	DC_TAG_HANDLE tag_handle,
	DC_TAG * tag_value )
{
	int rc;
	
	memset( tag_value, 0, sizeof( *tag_value ) );
	rc = dc_tag_read( &tag_handle, tag_value, 1 );
	if ( 0 == rc )
	{
		print_tag_value( tag_name, tag_value );
	}
	return rc;
}

int main ( int argc, char * argv[] )
{
	DC_TAG_HANDLE tag_handle_src;
	DC_TAG_HANDLE tag_handle_dst;
	DC_TAG tag_value_src;
	DC_TAG tag_value_dst;
	int rc;
	
	if ( argc != 3 )
	{
		printf( "usage: dc_tag_copy <source tag name> <destination tag name>\n" );
		return -1;
	}

	// initialize the DCTag SDK
	rc = dc_tag_init( 2, NULL, NULL );
	if ( rc != 0 )
	{
		printf( "dc_tag_init failed! error = %d\n", rc );
		return -2;
	}

	// open the tag
	tag_handle_src = dc_tag_open( argv[ 1 ], NULL );
	if ( NULL == tag_handle_src )
	{
		printf( "fail to open tag: %s\n", argv[ 1 ] );
		dc_tag_uninit();
		return -3;
	}

	tag_handle_dst = dc_tag_open( argv[ 2 ], NULL );
	if ( NULL == tag_handle_dst )
	{
		printf( "fail to open tag: %s\n", argv[ 2 ] );
		dc_tag_close( tag_handle_src );
		dc_tag_uninit();
		return -3;
	}

	// read tag
	printf( "Before copy:\n" );
	rc = read_tag( argv[1], tag_handle_src, &tag_value_src );
	if ( rc != 0 )
	{
		printf( "fail to read source tag value! error = %d\n", rc );
		dc_tag_close( tag_handle_src );
		dc_tag_close( tag_handle_dst );
		dc_tag_uninit();
		return -4;
	}

	rc = read_tag( argv[2], tag_handle_dst, &tag_value_dst );
	if ( rc != 0 )
	{
		printf( "fail to read destination tag value! error = %d\n", rc );
		dc_tag_close( tag_handle_src );
		dc_tag_close( tag_handle_dst );
		dc_tag_uninit();
		return -4;
	}

	if ( tag_value_dst.quality < 0 )
	{
		printf( "we can't write the destination tag with the bad quality value: %d\n", tag_value_dst.quality );
		dc_tag_close( tag_handle_src );
		dc_tag_close( tag_handle_dst );
		dc_tag_uninit();
		return -5;
	}

	// write tag
	rc = dc_tag_write( &tag_handle_dst, &tag_value_src.value, 1 );
	sleep(3);
	printf( "\n" );
	printf( "Write %s with value = %.16lf return: %d\n", argv[2], tag_value_src.value, rc );

	printf( "\n" );
	printf( "After copy:\n" );
	rc = read_tag( argv[1], tag_handle_src, &tag_value_src );
	if ( rc != 0 )
	{
		printf( "fail to read source tag value! error = %d\n", rc );
		dc_tag_close( tag_handle_src );
		dc_tag_close( tag_handle_dst );
		dc_tag_uninit();
		return -6;
	}

	rc = read_tag( argv[2], tag_handle_dst, &tag_value_dst );
	if ( rc != 0 )
	{
		printf( "fail to read destination tag value! error = %d\n", rc );
		dc_tag_close( tag_handle_src );
		dc_tag_close( tag_handle_dst );
		dc_tag_uninit();
		return -6;
	}

	// close tag handles
	dc_tag_close( tag_handle_src );
	dc_tag_close( tag_handle_dst );

	// uninitialize the SDK
	dc_tag_uninit();
	
	return 0;
}
